﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Shell;
using System.Collections.Generic;

namespace LWP.NavigationSamples
{
    //HOMEBUTTON STEP 0:  
    public class HomeNavigationHelper
    {

        static HomeNavigationHelper()
        {
            //Subscribe to tombstoning events so we can save the history. 
            PhoneApplicationService.Current.Deactivated += new EventHandler<DeactivatedEventArgs>(OnPhoneActivationServiceDeactivated);
            PhoneApplicationService.Current.Activated += new EventHandler<ActivatedEventArgs>(OnPhoneApplicationServiceActivated);
            PhoneApplicationService.Current.Launching += new EventHandler<LaunchingEventArgs>(OnPhoneApplicationServiceLaunching);
        }

        #region Properties & events

        public static HomeNavigationHelper Instance
        {
            get
            {
                if (_instance == null)
                {
                    _instance = new HomeNavigationHelper();
                }
                return _instance;

            }
        }

        public bool IsInitialized
        {
            get;
            private set;
        }

        public bool IsLoaded
        {
            get;
            private set;
        }

        public bool AutoJournal
        {
            get
            {
                return _autoJournal;
            }
            set
            {
                if (_autoJournal != value)
                {
                    _autoJournal = value;

                    if (!AutoJournal)
                        StopJournaling();
                }
            }
        }

        public bool HideFrameonHomeNav { get; set; }
        public bool ValidateUriBeforeHomeNav { get; set; }
        public bool IsDoingHomeNavigation
        {
            get;
            private set;
        }

        public bool IsNavigationInProgress
        {
            get;
            private set;
        }

        public event EventHandler<HomeNavigationStartedEventArgs> HomeNavigationStarting;
        public event EventHandler<HomeNavigationCompletedEventArgs> HomeNavigationCompleted;

        #endregion      

        #region Public methods
        public bool Initialize(PhoneApplicationFrame frame, bool autoJournal, bool hideFrameOnNav, bool validateUriBeforeNav)
        {
            if (!IsInitialized)
            {
                frame.Navigated += new System.Windows.Navigation.NavigatedEventHandler(frame_Navigated);
                frame.Navigating += new System.Windows.Navigation.NavigatingCancelEventHandler(frame_Navigating);
                frame.NavigationStopped += new NavigationStoppedEventHandler(frame_NavigationStopped);
                _frame = frame;
                this.AutoJournal = autoJournal;
                this.HideFrameonHomeNav = hideFrameOnNav;
                this.ValidateUriBeforeHomeNav = validateUriBeforeNav;
                IsInitialized = true;

                return true;
            }
            return false;
        }

        //HOMEBUTTON STEP 4:  
        public void SafeNavigate(Uri targetUri, bool preventCircles = false)
        {
            if (IsNavigationInProgress)
            {
                if (UriInProgress != targetUri)
                {
                    _frame.StopLoading();
                    System.Diagnostics.Debug.WriteLine("Stopped Loading a nav");
                }
                else
                {
                    System.Diagnostics.Debug.WriteLine("Ignore a double click ");
                    return;
                }
            }
            else if (targetUri == _frame.Source)
            {
                System.Diagnostics.Debug.WriteLine("Ignore fragment navigation");
                return;
            }


            if (preventCircles)
            {
                if (journalingStack.Contains(targetUri))
                    this.StartHomeNavigation(targetUri);
                else
                   _frame.Navigate(targetUri);

            }
            else
                _frame.Navigate(targetUri);
        }
        public void StartHomeNavigation(Uri targetUri)
        {
            OnStartHomeNavigation(targetUri);
        }

        public void StopHomeNavigation(bool completed, bool userInitiated)
        {
            OnStopHomeNavigation(completed, userInitiated);
        }
        #endregion

        #region State
        const string BackNavHistoryKey = "BackNavigationHelper.History";
        const string HideFrameOnBackNavKey = "BackNavigationHelper.HideFrameOnBackNav";
        const string AutoJournalKey = "BackNavigationHelper.AutoJournal";
        const string ValidateUriBeforeBackNavKey = "BackNavigationHelper.ValidateUriBeforeBackNav";

        static void OnPhoneApplicationServiceLaunching(object sender, LaunchingEventArgs e)
        {



        }



        void SaveState(string key, object value)
        {
            PhoneApplicationService.Current.State[key] = value;
        }

        static void OnPhoneActivationServiceDeactivated(object sender, DeactivatedEventArgs e)
        {
            Instance.Save();
        }

        void Save()
        {
            SaveState(BackNavHistoryKey, journalingStack.ToArray());
            SaveState(HideFrameOnBackNavKey, HideFrameonHomeNav);
            SaveState(ValidateUriBeforeBackNavKey, ValidateUriBeforeHomeNav);
            SaveState(AutoJournalKey, AutoJournal);
        }

        bool isActivating = false;
        void Load()
        {

            /* TODO: Right now we allow duplication of Loading when a chooser that does not 
             tombstone but does deactivate  
             */
            System.Diagnostics.Debug.Assert(
                (IsLoaded == true && PhoneApplicationService.Current.StartupMode == StartupMode.Activate) ||
                  IsLoaded == false);

            if (PhoneApplicationService.Current.StartupMode == StartupMode.Activate)
            {

                System.Diagnostics.Debug.WriteLine("Activated");
                Uri[] historyentries = LoadState<Uri[]>(BackNavHistoryKey, null);
                if (historyentries != null)
                {
                    journalingStack.Clear();
                    for (int x = historyentries.Length - 1; x >= 0; x--)
                        journalingStack.Push(historyentries[x]);
                }

                Instance.HideFrameonHomeNav = LoadState<bool>(HideFrameOnBackNavKey, true);
                Instance.AutoJournal = LoadState<bool>(AutoJournalKey, true);
                Instance.ValidateUriBeforeHomeNav = LoadState<bool>(ValidateUriBeforeBackNavKey, false);
            }
#if DEBUG
            else
                System.Diagnostics.Debug.WriteLine("Launched");
#endif
            this.IsLoaded = true;
        }

        T LoadState<T>(string key, T defaultValue)
        {
            if (PhoneApplicationService.Current.State.ContainsKey(key))
            {
                return (T)PhoneApplicationService.Current.State[key];
            }
            return defaultValue;

        }

        private bool IsTombstoned { get; set; }

        static void OnPhoneApplicationServiceActivated(object sender, ActivatedEventArgs e)
        {
            Instance.IsTombstoned = true;

            //TODO:   This checks if this was truly a tombstone.. 
            //  Some choosers do not tombstone the app in full..  they do deactivate but 
            if (PhoneApplicationService.Current.StartupMode == StartupMode.Activate)
                Instance.Load();
        }

        #endregion


        #region private and virtual methods

        void frame_NavigationStopped(object sender, NavigationEventArgs e)
        {
            if (IsDoingHomeNavigation)
                StopHomeNavigation(false, true);

            IsNavigationInProgress = false;
        }

        void StopJournaling()
        {
            journalingStack.Clear();
        }

        public virtual void OnStopHomeNavigation(bool completed, bool userInitiated)
        {

            EventHandler<HomeNavigationCompletedEventArgs> eh = this.HomeNavigationCompleted;
            HomeNavigationCompletedEventArgs args = new HomeNavigationCompletedEventArgs { Destination = _homeNavigationUri, NavigationCompleted = completed, UserCancelled = userInitiated };
            if (eh != null)
                eh(this, args);

            IsDoingHomeNavigation = false;
            _homeNavigationUri = null;

            if (HideFrameonHomeNav)
                _frame.Opacity = frameOpacity;

 

        }

        public Uri UriInProgress
        {
            get;
            internal set;
        }


        //HOMEBUTTON STEP 1:  
        public virtual bool OnStartHomeNavigation(Uri targetUri)
        {
            if (AutoJournal && ValidateUriBeforeHomeNav)
            {
                if (!journalingStack.Contains(targetUri))
                    return false;
            }

            EventHandler<HomeNavigationStartedEventArgs> start = this.HomeNavigationStarting;
            HomeNavigationStartedEventArgs args = new HomeNavigationStartedEventArgs { Destination = targetUri };
            if (start != null)
                start(this, args);

            // if no page objected to a back nav then do it. 
            if (!args.Cancel)
            {
                IsDoingHomeNavigation = true;
                _homeNavigationUri = targetUri;

                if (HideFrameonHomeNav)
                {
                    frameOpacity = _frame.Opacity;
                    _frame.Opacity = 0;
                }

                _frame.GoBack();
            }

            return !args.Cancel;
        }



        void frame_Navigating(object sender, System.Windows.Navigation.NavigatingCancelEventArgs e)
        {

            currentDirection = e.NavigationMode;

            UriInProgress = e.Uri;
#if DEBUG
            switch (e.NavigationMode)
            {
                case NavigationMode.Back:
                    break;
                case NavigationMode.New:
                    break;
                case NavigationMode.Refresh:
                    System.Diagnostics.Debug.Assert(false, "trying to learn when this happens");
                    break;
                case NavigationMode.Forward:
                    System.Diagnostics.Debug.Assert(false, "trying to learn when this happens");
                    break;
            }
#endif

            this.IsNavigationInProgress = true;
        }


        //HOMEBUTTON STEP 2:  
        void frame_Navigated(object sender, System.Windows.Navigation.NavigationEventArgs e)
        {

            //HOMEBUTTON STEP 2.1:   Autojournaling is adding stuff to the stack so we can validate later that a URL is in the stack

            if (AutoJournal)
            {
                if (currentDirection == NavigationMode.Back)
                {
                    if (journalingStack.Count > 0)
                    {

                        if (!IsTombstoned)
                        {
                            journalingStack.Pop();
                        }
                        else
                            IsTombstoned = false;
#if DEBUG
                        if (journalingStack.Count > 0)
                        {
                            Uri top = journalingStack.Peek();
                            System.Diagnostics.Debug.Assert(top == e.Uri);
                        }
#endif
                    }
                    // we have reached pages that were not on our stack. This likely won't happen 
                    // for real apps, but happens for demos when you turn journaling on/off.. 
                    else
                    {
                        journalingStack.Push(e.Uri);
                    }
                }

                if (currentDirection == NavigationMode.New)
                {
                    // SKIP the tombstoning URLs 
                    if (e.Uri == new Uri("app://external", UriKind.RelativeOrAbsolute))
                    {
                        IsTombstoned = true;
                        System.Diagnostics.Debug.WriteLine("*** External *** ");
                    }
                    else

                        journalingStack.Push(e.Uri);
                }
            }

            //HOMEBUTTON STEP 2.2:  Check for back navigation and continue the sequence
            if (IsDoingHomeNavigation)
            {
                if (e.Uri != _homeNavigationUri && _frame.CanGoBack)
                {
                    _frame.GoBack();
                    return;
                }
                else
                {
                    StopHomeNavigation(true, false);
                }

            }
            IsNavigationInProgress = false;
        }

        #endregion 

        #region members
        private PhoneApplicationFrame _frame;
        private Uri _homeNavigationUri;
        private static HomeNavigationHelper _instance;
        bool _autoJournal;
        System.Collections.Generic.Stack<Uri> journalingStack = new System.Collections.Generic.Stack<Uri>();
        public static Settings DemoSettings = new Settings();
        NavigationMode currentDirection;
        double frameOpacity;


        public class Settings
        {
            public bool HideAppBarOnNavigate { get; set; }
            public bool PreventLoopCreation { get; set; }
        }
        #endregion
    }


    public class HomeNavigationStartedEventArgs : EventArgs
    {
        public Uri Destination;
        public bool Cancel;
    }

    public class HomeNavigationCompletedEventArgs : EventArgs
    {
        public Uri Destination;
        public bool NavigationCompleted;
        public bool UserCancelled;
    }
}
