﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;

namespace LWP.NavigationSamples.HomeNavigationSamplePages
{
    public partial class ItemPage : PhoneApplicationPage
    {

        WeakEventListener<ItemPage, object, HomeNavigationCompletedEventArgs> navigationCompletedListener;
        WeakEventListener<ItemPage, object, HomeNavigationStartedEventArgs> navigationStartedListener; 


        public ItemPage()
        {
            InitializeComponent();


         

#if DEBUG 
            instanceCount++;
#endif 
        }

#if DEBUG 
        static int instanceCount; 
        ~ItemPage()
        {
            System.Diagnostics.Debug.WriteLine("~ItemPage: {0} instances left", instanceCount-- );  
        } 

#endif 

        bool appBarVisibility; 
        void Instance_HomeNavigationCompleted(object sender, HomeNavigationCompletedEventArgs e)
        {
            if ( HomeNavigationHelper.DemoSettings.HideAppBarOnNavigate ) 
                this.ApplicationBar.IsVisible = appBarVisibility;  
        }

        void Instance_BackNavigationStarting(object sender, HomeNavigationStartedEventArgs e)
        {
            if (HomeNavigationHelper.DemoSettings.HideAppBarOnNavigate)
            {
                appBarVisibility = this.ApplicationBar.IsVisible;
                this.ApplicationBar.IsVisible = false;
            } 
        }



        public static int Next;
        public int instanceId;
        int level = 0;
        int item = 0;
        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {

            if (!HomeNavigationHelper.Instance.IsDoingHomeNavigation)
            {
                base.OnNavigatedTo(e);
                level = int.Parse(NavigationContext.QueryString["level"]);
                if (NavigationContext.QueryString.ContainsKey("item"))
                    item = int.Parse(NavigationContext.QueryString["item"]);


                this.PageTitle.Text = "level " + level + ",item=" + item;

                if (level > 0)
                {
                    upLevel.IsEnabled = true;
                }

                string appbarstring;

                if (NavigationContext.QueryString.TryGetValue("appbar", out appbarstring))
                {
                    bool showAppbarFlag;
                    bool.TryParse(appbarstring, out showAppbarFlag);
                    this.ApplicationBar.IsVisible = showAppbarFlag;
                    this.showAppBar.IsChecked = showAppbarFlag;

                    /* 
                     *  If we are showing applicationbar and are not listening for BackNavigationHelper's navigation events, we should subscribe
                     *  SAMPLE NOTE:  
                     *  This makes huge assumptions that Appbar will not be enabled via page interactions 
                     *  I did it that way to demonstrate that the navigation events are pay per play.. but I assume most apps will subscribe to
                     *  this all the time */


                    //HOMEBUTTON STEP 3:  
                    if (showAppbarFlag && navigationCompletedListener == null)
                    {
                        navigationCompletedListener = new WeakEventListener<ItemPage, object, HomeNavigationCompletedEventArgs>(this);
                        navigationCompletedListener.OnEventAction = (instance, source, args) => instance.Instance_HomeNavigationCompleted(source, args);
                        navigationCompletedListener.OnDetachAction = (listener) => HomeNavigationHelper.Instance.HomeNavigationCompleted -= listener.OnEvent;
                        HomeNavigationHelper.Instance.HomeNavigationCompleted += navigationCompletedListener.OnEvent;


                        navigationStartedListener = new WeakEventListener<ItemPage, object, HomeNavigationStartedEventArgs>(this);
                        navigationStartedListener.OnEventAction = (instance, source, args) => instance.Instance_BackNavigationStarting(source, args);
                        navigationStartedListener.OnDetachAction = (listener) => HomeNavigationHelper.Instance.HomeNavigationStarting -= listener.OnEvent;
                        HomeNavigationHelper.Instance.HomeNavigationStarting += navigationStartedListener.OnEvent;
                    }

                    /* 
                     * This just detaches the event when the applicationbar is not shown, since we then would not need callbacks
                     */
                    if (!showAppbarFlag && navigationCompletedListener != null)
                    {
                        navigationCompletedListener.Detach();
                        navigationCompletedListener = null;

                        navigationStartedListener.Detach();
                        navigationStartedListener = null;
                    }
                }
            }
            else
            {
                if (this.ApplicationBar.IsVisible)
                {
                    System.Diagnostics.Debug.WriteLine("Hiding appbar in tombstone back"); 
                    appBarVisibility = this.ApplicationBar.IsVisible;
                    this.ApplicationBar.IsVisible = HomeNavigationHelper.DemoSettings.HideAppBarOnNavigate;
                }
            } 


        }


        void GoHome(object sender, RoutedEventArgs args)
        {
            HomeNavigationHelper.Instance.StartHomeNavigation(new Uri("/HomeNavigationSamplePages/LevelPage.xaml?level=0", UriKind.Relative));
        }

        void GoUp(object sender, RoutedEventArgs args)
        {
            Uri targetUri = new Uri(
                string.Format("/HomeNavigationSamplePages/LevelPage.xaml?level={0}&appbar={1}", level - 1,
                    showAppBar.IsChecked.Value), UriKind.Relative); 

            if ( HomeNavigationHelper.DemoSettings.PreventLoopCreation ) 
            {
                HomeNavigationHelper.Instance.SafeNavigate(targetUri, true); 
            } 
            else 
                HomeNavigationHelper.Instance.StartHomeNavigation(targetUri);
        }

        void GoDown(object u, RoutedEventArgs u2)
        {
             
            NavigationService.Navigate(new Uri(
                string.Format("/HomeNavigationSamplePages/LevelPage.xaml?level={0}&appbar={1}", level + 1,
                showAppBar.IsChecked.Value), UriKind.Relative));
        }

        void GoBack(object u, RoutedEventArgs u2)
        {
            if (NavigationService.CanGoBack)
                NavigationService.GoBack();
        }

        void GoToItem(object u, RoutedEventArgs u2)
        {
            Uri targetUri = new Uri(
                string.Format("/HomeNavigationSamplePages/ItemPage.xaml?level={0}&item={1}&appbar={2}", level, item + 1,
                showAppBar.IsChecked.Value), UriKind.Relative);

            HomeNavigationHelper.Instance.SafeNavigate(targetUri);  
        }


        void OnLaunchChooser(object u, RoutedEventArgs u2)
        {
            Microsoft.Phone.Tasks.PhotoChooserTask chooser = new Microsoft.Phone.Tasks.PhotoChooserTask();
            chooser.ShowCamera = true;
            chooser.Show(); 
        } 
    }
}