﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;

namespace LWP.NavigationSamples.HomeNavigationSamplePages
{
    public partial class LevelPage : PhoneApplicationPage
    {

        WeakEventListener<LevelPage, object, HomeNavigationCompletedEventArgs> navigationCompletedListener;
        WeakEventListener<LevelPage, object, HomeNavigationStartedEventArgs> navigationStartedListener; 

        public LevelPage()
        {
            InitializeComponent();          

            //BackNavigationHelper.Instance.BackNavigationStarted += new EventHandler<BackNavigationStartedEventArgs>(Instance_BackNavigationStarted);
            //BackNavigationHelper.Instance.BackNavigationCompleted += new EventHandler<BackNavigationCompletedEventArgs>(Instance_BackNavigationCompleted);

            navigationCompletedListener = new WeakEventListener<LevelPage,object,HomeNavigationCompletedEventArgs>(this); 
            navigationCompletedListener.OnEventAction = (instance, source, args ) => instance.Instance_BackNavigationCompleted(source, args ); 
            navigationCompletedListener.OnDetachAction = (listener) => HomeNavigationHelper.Instance.HomeNavigationCompleted -= listener.OnEvent ; 
            HomeNavigationHelper.Instance.HomeNavigationCompleted += navigationCompletedListener.OnEvent ;


            navigationStartedListener = new WeakEventListener<LevelPage, object, HomeNavigationStartedEventArgs>(this);
            navigationStartedListener.OnEventAction = (instance, source, args) => instance.Instance_BackNavigationStarted(source, args);
            navigationStartedListener.OnDetachAction = (listener) => HomeNavigationHelper.Instance.HomeNavigationStarting -= listener.OnEvent;
            HomeNavigationHelper.Instance.HomeNavigationStarting += navigationStartedListener.OnEvent; 



#if DEBUG
            instanceCount++;
#endif 
        }

#if DEBUG 
        static int instanceCount; 
        ~LevelPage()
        {
            System.Diagnostics.Debug.WriteLine("~Level: {0} instances left", instanceCount-- ); 
        } 

#endif 


        bool appBarVisibility;
        void Instance_BackNavigationCompleted(object sender, HomeNavigationCompletedEventArgs e)
        {
            if (HomeNavigationHelper.DemoSettings.HideAppBarOnNavigate)
                this.ApplicationBar.IsVisible = appBarVisibility;
        }

        void Instance_BackNavigationStarted(object sender, HomeNavigationStartedEventArgs e)
        {
            if (HomeNavigationHelper.DemoSettings.HideAppBarOnNavigate)
            {
                appBarVisibility = this.ApplicationBar.IsVisible;
                this.ApplicationBar.IsVisible = false;
            } 
        }

        public static int Next;
        public int instanceId; 
        int level = 0;
        int item = 0; 
        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            if (!HomeNavigationHelper.Instance.IsDoingHomeNavigation)
            {
                base.OnNavigatedTo(e);
                if (NavigationContext.QueryString.ContainsKey("level"))
                    level = int.Parse(NavigationContext.QueryString["level"]);
                if (NavigationContext.QueryString.ContainsKey("item"))
                    item = int.Parse(NavigationContext.QueryString["item"]);

                string appbarstring;
                if (NavigationContext.QueryString.TryGetValue("appbar", out appbarstring))
                {
                    bool showAppbarFlag;
                    bool.TryParse(appbarstring, out showAppbarFlag);
                    this.ApplicationBar.IsVisible = showAppbarFlag;
                    this.showAppBar.IsChecked = showAppbarFlag;
                }

                this.PageTitle.Text = "level " + level;

                if (level > 0)
                {
                    upLevel.IsEnabled = true;
                }
            }
            else
            {
                if (this.ApplicationBar.IsVisible)
                {
                    System.Diagnostics.Debug.WriteLine("Hiding appbar in tombstone back"); 
                    appBarVisibility = this.ApplicationBar.IsVisible; 
                    this.ApplicationBar.IsVisible = HomeNavigationHelper.DemoSettings.HideAppBarOnNavigate;
                }
            } 
        }


        void GoHome(object sender, RoutedEventArgs args)
        {
            HomeNavigationHelper.Instance.StartHomeNavigation(new Uri("/HomeNavigationSamplePages/LevelPage.xaml?level=0", UriKind.Relative)); 
        }

        void GoUp(object sender, RoutedEventArgs args)
        {
            Uri targetUri = new Uri(
                string.Format("/HomeNavigationSamplePages/LevelPage.xaml?level={0}&appbar={1}", level - 1, 
                    showAppBar.IsChecked.Value), UriKind.Relative);

            HomeNavigationHelper.Instance.SafeNavigate(targetUri); 
        } 

        void GoDown ( object u, RoutedEventArgs u2) 
        { 
            Uri targetUri = new Uri(
                string.Format("/HomeNavigationSamplePages/LevelPage.xaml?level={0}&appbar={1}", level + 1, 
                showAppBar.IsChecked.Value), UriKind.Relative);


            HomeNavigationHelper.Instance.SafeNavigate(targetUri);  
        }

        void GoBack(object u, RoutedEventArgs u2)
        {
            if ( NavigationService.CanGoBack ) 
                NavigationService.GoBack(); 
        }

        void GoToItem(object u, RoutedEventArgs u2)
        {
            Uri targetUri = new Uri(
                string.Format("/HomeNavigationSamplePages/ItemPage.xaml?level={0}&item={1}&appbar={2}", level, item + 1,
                showAppBar.IsChecked.Value), UriKind.Relative);

            HomeNavigationHelper.Instance.SafeNavigate(targetUri); 

            
        } 
    }
}